@@ -152,6 +152,9 @@ typedef jit_int32_t		jit_fpr_t;
 #define JIT_V_NUM		jit_v_num()
 #define JIT_F_NUM		jit_f_num()
 
+#define JIT_DISABLE_DATA	1	/* force synthesize of constants */
+#define JIT_DISABLE_NOTE	2	/* disable debug info generation */
+
 #define jit_class_chk		0x02000000	/* just checking */
 #define jit_class_arg		0x08000000	/* argument register */
 #define jit_class_sav		0x10000000	/* callee save */
@@ -174,6 +177,7 @@ typedef enum {
 #define jit_note(u, v)		_jit_note(_jit, u, v)
 #define jit_label()		_jit_label(_jit)
 #define jit_forward()		_jit_forward(_jit)
+#define jit_indirect()		_jit_indirect(_jit)
 #define jit_link(u)		_jit_link(_jit,u)
     jit_code_note,		jit_code_label,
 
@@ -841,6 +845,7 @@ extern jit_node_t *_jit_name(jit_state_t*, char*);
 extern jit_node_t *_jit_note(jit_state_t*, char*, int);
 extern jit_node_t *_jit_label(jit_state_t*);
 extern jit_node_t *_jit_forward(jit_state_t*);
+extern jit_node_t *_jit_indirect(jit_state_t*);
 extern void _jit_link(jit_state_t*, jit_node_t*);
 
 extern void _jit_prolog(jit_state_t*);
@@ -885,6 +890,16 @@ extern void _jit_patch(jit_state_t*, jit_node_t*);
 extern void _jit_patch_at(jit_state_t*, jit_node_t*, jit_node_t*);
 #define jit_patch_abs(u,v)	_jit_patch_abs(_jit,u,v)
 extern void _jit_patch_abs(jit_state_t*, jit_node_t*, jit_pointer_t);
+#define jit_realize()		_jit_realize(_jit)
+extern void _jit_realize(jit_state_t*);
+#define jit_get_code(u)		_jit_get_code(_jit,u)
+extern jit_pointer_t _jit_get_code(jit_state_t*, jit_word_t*);
+#define jit_set_code(u,v)	_jit_set_code(_jit,u,v)
+extern void _jit_set_code(jit_state_t*, jit_pointer_t, jit_word_t);
+#define jit_get_data(u,v)	_jit_get_data(_jit,u,v)
+extern jit_pointer_t _jit_get_data(jit_state_t*, jit_word_t*, jit_word_t*);
+#define jit_set_data(u,v,w)	_jit_set_data(_jit,u,v,w)
+extern void _jit_set_data(jit_state_t*, jit_pointer_t, jit_word_t, jit_word_t);
 #define jit_emit()		_jit_emit(_jit)
 extern jit_pointer_t _jit_emit(jit_state_t*);
 
