@@ -103,6 +103,6 @@ typedef enum {
 #define JIT_NOREG		_NOREG
 } jit_reg_t;
 
-typedef jit_int64_t		jit_regset_t;
+typedef jit_uint64_t		jit_regset_t;
 
 #endif /* _jit_ppc_h */
