@@ -80,6 +80,10 @@ static void _bmp_clr(jit_state_t*, jit_word_t) maybe_unused;
 #define bmp_tst(bit)			_bmp_tst(_jit, bit)
 static jit_bool_t _bmp_tst(jit_state_t*, jit_word_t);
 
+#define jit_dataset()			_jit_dataset(_jit)
+static void
+_jit_dataset(jit_state_t *_jit);
+
 #define jit_setup(block)		_jit_setup(_jit, block)
 static void
 _jit_setup(jit_state_t *_jit, jit_block_t *block);
@@ -496,10 +500,15 @@ jit_regset_scan1(jit_regset_t *set, jit_int32_t offset)
 unsigned long
 jit_regset_scan1(jit_regset_t *set, jit_int32_t offset)
 {
+    jit_regset_t       mask;
     assert(offset >= 0 && offset <= 63);
-    for (; offset < 64; offset++) {
-	if (*set & (1LL << offset))
-	    return (offset);
+    if ((mask = *set >> offset)) {
+	for (;;) {
+	    if (mask & 1)
+		return (offset);
+	    mask >>= 1;
+	    ++offset;
+	}
     }
     return (ULONG_MAX);
 }
@@ -509,7 +518,7 @@ void
 _jit_save(jit_state_t *_jit, jit_int32_t reg)
 {
     reg = jit_regno(reg);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     _jitc->spill[reg] = jit_new_node_w(jit_code_save, reg);
 }
 
@@ -519,7 +528,7 @@ _jit_load(jit_state_t *_jit, jit_int32_t reg)
     jit_node_t		*node;
 
     reg = jit_regno(reg);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     assert(_jitc->spill[reg]);
     node = jit_new_node_w(jit_code_load, reg);
     /* create a path to flag the save/load is not required */
@@ -543,7 +552,10 @@ _jit_address(jit_state_t *_jit, jit_node_t *node)
 {
     assert(_jitc->done);
     assert(node &&
-	   (node->code == jit_code_note || node->code == jit_code_name));
+	   /* If a node type that is documented to be a fixed marker */
+	   (node->code == jit_code_note || node->code == jit_code_name ||
+	    /* If another special fixed marker, returned by jit_indirect() */
+	    (node->code == jit_code_label && (node->flag & jit_flag_use))));
     return ((jit_pointer_t)node->u.w);
 }
 
@@ -554,7 +566,7 @@ _jit_data(jit_state_t *_jit, jit_pointer_t data,
     jit_word_t		 key;
     jit_node_t		*node;
 
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
 
     /* Ensure there is space even if asking for a duplicate */
     if (((_jitc->data.offset + 7) & -8) + length > _jit->data.length) {
@@ -562,10 +574,10 @@ _jit_data(jit_state_t *_jit, jit_pointer_t data,
 
 	size = (_jit->data.length + length + 4096) & - 4095;
 	assert(size >= _jit->data.length);
-	if (_jit->data.ptr == NULL)
-	    jit_alloc((jit_pointer_t *)&_jit->data.ptr, size);
+	if (_jitc->data.ptr == NULL)
+	    jit_alloc((jit_pointer_t *)&_jitc->data.ptr, size);
 	else
-	    jit_realloc((jit_pointer_t *)&_jit->data.ptr,
+	    jit_realloc((jit_pointer_t *)&_jitc->data.ptr,
 			_jit->data.length, size);
 	_jit->data.length = size;
     }
@@ -577,7 +589,7 @@ _jit_data(jit_state_t *_jit, jit_pointer_t data,
     node = _jitc->data.table[key];
     for (; node; node = node->next) {
 	if (node->v.w == length &&
-	    memcmp(_jit->data.ptr + node->u.w, data, length) == 0)
+	    memcmp(_jitc->data.ptr + node->u.w, data, length) == 0)
 	    break;
     }
 
@@ -600,7 +612,7 @@ _jit_data(jit_state_t *_jit, jit_pointer_t data,
 	}
 	node->u.w = _jitc->data.offset;
 	node->v.w = length;
-	memcpy(_jit->data.ptr + _jitc->data.offset, data, length);
+	memcpy(_jitc->data.ptr + _jitc->data.offset, data, length);
 	_jitc->data.offset += length;
 
 	node->next = _jitc->data.table[key];
@@ -622,7 +634,7 @@ _jit_data(jit_state_t *_jit, jit_pointer_t data,
 		temp = _jitc->data.table[i];
 		for (; temp; temp = next) {
 		    next = temp->next;
-		    key = hash_data(_jit->data.ptr + temp->u.w, temp->v.w) &
+		    key = hash_data(_jitc->data.ptr + temp->u.w, temp->v.w) &
 			  ((_jitc->data.size << 1) - 1);
 		    temp->next = hash[key];
 		    hash[key] = temp;
@@ -879,29 +891,31 @@ _jit_clear_state(jit_state_t *_jit)
 void
 _jit_destroy_state(jit_state_t *_jit)
 {
-    munmap(_jit->code.ptr, _jit->code.length);
-    munmap(_jit->data.ptr, _jit->data.length);
+    if (!_jit->user_code)
+	munmap(_jit->code.ptr, _jit->code.length);
+    if (!_jit->user_data)
+	munmap(_jit->data.ptr, _jit->data.length);
     jit_free((jit_pointer_t *)&_jit);
 }
 
 jit_node_t *
 _jit_new_node(jit_state_t *_jit, jit_code_t code)
 {
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     return (link_node(new_node(code)));
 }
 
 jit_node_t *
 _jit_new_node_no_link(jit_state_t *_jit, jit_code_t code)
 {
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     return (new_node(code));
 }
 
 void
 _jit_link_node(jit_state_t *_jit, jit_node_t *node)
 {
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     link_node(node);
 }
 
@@ -910,7 +924,7 @@ _jit_new_node_w(jit_state_t *_jit, jit_code_t code,
 		jit_word_t u)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.w = u;
     return (link_node(node));
 }
@@ -920,7 +934,7 @@ _jit_new_node_p(jit_state_t *_jit, jit_code_t code,
 		jit_pointer_t u)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.p = u;
     return (link_node(node));
 }
@@ -930,7 +944,7 @@ _jit_new_node_ww(jit_state_t *_jit, jit_code_t code,
 		 jit_word_t u, jit_word_t v)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.w = u;
     node->v.w = v;
     return (link_node(node));
@@ -955,7 +969,7 @@ _jit_new_node_wf(jit_state_t *_jit, jit_code_t code,
 		 jit_word_t u, jit_float32_t v)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.w = u;
     node->v.f = v;
     return (link_node(node));
@@ -966,7 +980,7 @@ _jit_new_node_wd(jit_state_t *_jit, jit_code_t code,
 		 jit_word_t u, jit_float64_t v)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.w = u;
     node->v.d = v;
     return (link_node(node));
@@ -977,7 +991,7 @@ _jit_new_node_www(jit_state_t *_jit, jit_code_t code,
 		  jit_word_t u, jit_word_t v, jit_word_t w)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.w = u;
     node->v.w = v;
     node->w.w = w;
@@ -990,7 +1004,7 @@ _jit_new_node_qww(jit_state_t *_jit, jit_code_t code,
 		  jit_word_t v, jit_word_t w)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     assert(l != h);
     node->u.q.l = l;
     node->u.q.h = h;
@@ -1004,7 +1018,7 @@ _jit_new_node_wwf(jit_state_t *_jit, jit_code_t code,
 		  jit_word_t u, jit_word_t v, jit_float32_t w)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.w = u;
     node->v.w = v;
     node->w.f = w;
@@ -1016,7 +1030,7 @@ _jit_new_node_wwd(jit_state_t *_jit, jit_code_t code,
 		  jit_word_t u, jit_word_t v, jit_float64_t w)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.w = u;
     node->v.w = v;
     node->w.d = w;
@@ -1028,7 +1042,7 @@ _jit_new_node_pww(jit_state_t *_jit, jit_code_t code,
 		  jit_pointer_t u, jit_word_t v, jit_word_t w)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.p = u;
     node->v.w = v;
     node->w.w = w;
@@ -1040,7 +1054,7 @@ _jit_new_node_pwf(jit_state_t *_jit, jit_code_t code,
 		  jit_pointer_t u, jit_word_t v, jit_float32_t w)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.p = u;
     node->v.w = v;
     node->w.f = w;
@@ -1052,7 +1066,7 @@ _jit_new_node_pwd(jit_state_t *_jit, jit_code_t code,
 		  jit_pointer_t u, jit_word_t v, jit_float64_t w)
 {
     jit_node_t		*node = new_node(code);
-    assert(!_jitc->emit);
+    assert(!_jitc->realize);
     node->u.p = u;
     node->v.w = v;
     node->w.d = w;
@@ -1078,6 +1092,17 @@ _jit_forward(jit_state_t *_jit)
     return (jit_new_node_no_link(jit_code_label));
 }
 
+jit_node_t *
+_jit_indirect(jit_state_t *_jit)
+{
+    jit_node_t		*node;
+
+    node = jit_label();
+    node->flag |= jit_flag_use;
+
+    return (node);
+}
+
 void
 _jit_link(jit_state_t *_jit, jit_node_t *node)
 {
@@ -1397,15 +1422,11 @@ _jit_patch_at(jit_state_t *_jit, jit_node_t *instr, jit_node_t *label)
 void
 _jit_optimize(jit_state_t *_jit)
 {
-    jit_uint8_t		*ptr;
     jit_bool_t		 jump;
     jit_int32_t		 mask;
     jit_node_t		*node;
     jit_block_t		*block;
     jit_word_t		 offset;
-#if defined(__sgi)
-    int			 mmap_fd;
-#endif
 
     _jitc->function = NULL;
 
@@ -1514,38 +1535,6 @@ _jit_optimize(jit_state_t *_jit)
 		break;
 	}
     }
-
-    /* ensure it is aligned */
-    _jitc->data.offset = (_jitc->data.offset + 7) & -8;
-
-    /* create read only data buffer */
-    _jit->data.length = (_jitc->data.offset +
-			 /* reserve space for annotations */
-			 _jitc->note.size + 4095) & -4096;
-#if defined(__sgi)
-    mmap_fd = open("/dev/zero", O_RDWR);
-#endif
-    ptr = mmap(NULL, _jit->data.length,
-	       PROT_READ | PROT_WRITE,
-	       MAP_PRIVATE | MAP_ANON, mmap_fd, 0);
-    assert(ptr != MAP_FAILED);
-#if defined(__sgi)
-    close(mmap_fd);
-#endif
-    memcpy(ptr, _jit->data.ptr, _jitc->data.offset);
-    jit_free((jit_pointer_t *)&_jit->data.ptr);
-    _jit->data.ptr = ptr;
-
-    /* to be filled with note contents once offsets are known */
-    _jitc->note.base = ptr + _jitc->data.offset;
-    memset(_jitc->note.base, 0, _jit->data.length - _jitc->data.offset);
-
-    for (offset = 0; offset < _jitc->data.size; offset++) {
-	for (node = _jitc->data.table[offset]; node; node = node->next) {
-	    node->flag |= jit_flag_patch;
-	    node->u.w = (jit_word_t)(_jit->data.ptr + node->u.w);
-	}
-    }
 }
 
 void
@@ -1684,6 +1673,147 @@ _jit_regarg_clr(jit_state_t *_jit, jit_node_t *node, jit_int32_t value)
 	jit_regset_clrbit(&_jitc->regarg, jit_regno(node->w.w));
 }
 
+void
+_jit_realize(jit_state_t *_jit)
+{
+    assert(!_jitc->realize);
+    if (_jitc->function)
+	jit_epilog();
+    jit_optimize();
+    _jitc->realize = 1;
+
+    /* ensure it is aligned */
+    _jitc->data.offset = (_jitc->data.offset + 7) & -8;
+
+#if GET_JIT_SIZE
+    /* Heuristic to guess code buffer size */
+    _jitc->mult = 4;
+    _jit->code.length = _jitc->pool.length * 1024 * _jitc->mult;
+#else
+    _jit->code.length = jit_get_size();
+#endif
+}
+
+void
+_jit_dataset(jit_state_t *_jit)
+{
+    jit_uint8_t		*ptr;
+    jit_node_t		*node;
+    jit_word_t		 offset;
+#if defined(__sgi)
+    int			 mmap_fd;
+#endif
+
+    assert(!_jitc->dataset);
+    if (!_jit->user_data) {
+
+	/* create read only data buffer */
+	_jit->data.length = (_jitc->data.offset +
+			     /* reserve space for annotations */
+			     _jitc->note.size + 4095) & -4096;
+#if defined(__sgi)
+	mmap_fd = open("/dev/zero", O_RDWR);
+#endif
+	_jit->data.ptr = mmap(NULL, _jit->data.length,
+			      PROT_READ | PROT_WRITE,
+			      MAP_PRIVATE | MAP_ANON, mmap_fd, 0);
+	assert(_jit->data.ptr != MAP_FAILED);
+#if defined(__sgi)
+	close(mmap_fd);
+#endif
+    }
+
+    if (!_jitc->no_data)
+	memcpy(_jit->data.ptr, _jitc->data.ptr, _jitc->data.offset);
+
+    if (_jitc->no_note) {
+	_jit->note.length = 0;
+	_jitc->note.size = 0;
+    }
+    else {
+	_jitc->note.base = _jit->data.ptr;
+	if (!_jitc->no_data)
+	    _jitc->note.base += _jitc->data.offset;
+	memset(_jitc->note.base, 0, _jitc->note.size);
+    }
+
+    if (_jit->user_data)
+	/* Need the temporary hashed data until jit_emit is finished */
+	ptr = _jitc->no_data ? _jitc->data.ptr : _jit->data.ptr;
+    else {
+	ptr = _jit->data.ptr;
+	/* Temporary hashed data no longer required */
+	jit_free((jit_pointer_t *)&_jitc->data.ptr);
+    }
+
+    for (offset = 0; offset < _jitc->data.size; offset++) {
+	for (node = _jitc->data.table[offset]; node; node = node->next) {
+	    node->flag |= jit_flag_patch;
+	    node->u.w = (jit_word_t)(ptr + node->u.w);
+	}
+    }
+
+    _jitc->dataset = 1;
+}
+
+jit_pointer_t
+_jit_get_code(jit_state_t *_jit, jit_word_t *length)
+{
+    assert(_jitc->realize);
+    if (length) {
+	if (_jitc->done)
+	    /* If code already generated, return exact size of code */
+	    *length = _jit->pc.uc - _jit->code.ptr;
+	else
+	    /* Else return current size of the code buffer */
+	    *length = _jit->code.length;
+    }
+
+    return (_jit->code.ptr);
+}
+
+void
+_jit_set_code(jit_state_t *_jit, jit_pointer_t ptr, jit_word_t length)
+{
+    assert(_jitc->realize);
+    _jit->code.ptr = ptr;
+    _jit->code.length = length;
+    _jit->user_code = 1;
+}
+
+jit_pointer_t
+_jit_get_data(jit_state_t *_jit, jit_word_t *data_size, jit_word_t *note_size)
+{
+    assert(_jitc->realize);
+    if (data_size)
+	*data_size = _jitc->data.offset;
+    if (note_size)
+	*note_size = _jitc->note.size;
+    return (_jit->data.ptr);
+}
+
+void
+_jit_set_data(jit_state_t *_jit, jit_pointer_t ptr,
+	      jit_word_t length, jit_word_t flags)
+{
+    assert(_jitc->realize);
+    if (flags & JIT_DISABLE_DATA)
+	_jitc->no_data = 1;
+    else
+	assert(length >= _jitc->data.offset);
+    if (flags & JIT_DISABLE_NOTE)
+	_jitc->no_note = 1;
+    else {
+	if (flags & JIT_DISABLE_DATA)
+	    assert(length >= _jitc->note.size);
+	else
+	    assert(length >= _jitc->data.offset + _jitc->note.size);
+    }
+    _jit->data.ptr = ptr;
+    _jit->data.length = length;
+    _jit->user_data = 1;
+}
+
 jit_pointer_t
 _jit_emit(jit_state_t *_jit)
 {
@@ -1695,39 +1825,38 @@ _jit_emit(jit_state_t *_jit)
     int			 mmap_fd;
 #endif
 
-    if (_jitc->function)
-	jit_epilog();
-    jit_optimize();
+    if (!_jitc->realize)
+	jit_realize();
+
+    if (!_jitc->dataset)
+	jit_dataset();
 
     _jitc->emit = 1;
 
-#if GET_JIT_SIZE
-    /* Heuristic to guess code buffer size */
-    _jitc->mult = 4;
-    _jit->code.length = _jitc->pool.length * 1024 * _jitc->mult;
-#else
-    _jit->code.length = jit_get_size();
-#endif
-
+    if (!_jit->user_code) {
 #if defined(__sgi)
-    mmap_fd = open("/dev/zero", O_RDWR);
+	mmap_fd = open("/dev/zero", O_RDWR);
 #endif
-    _jit->code.ptr = mmap(NULL, _jit->code.length,
-			  PROT_EXEC | PROT_READ | PROT_WRITE,
-			  MAP_PRIVATE | MAP_ANON, mmap_fd, 0);
-    assert(_jit->code.ptr != MAP_FAILED);
+	_jit->code.ptr = mmap(NULL, _jit->code.length,
+			      PROT_EXEC | PROT_READ | PROT_WRITE,
+			      MAP_PRIVATE | MAP_ANON, mmap_fd, 0);
+	assert(_jit->code.ptr != MAP_FAILED);
+    }
     _jitc->code.end = _jit->code.ptr + _jit->code.length -
 	jit_get_max_instr();
     _jit->pc.uc = _jit->code.ptr;
 
     for (;;) {
 	if ((code = emit_code()) == NULL) {
+	    _jitc->patches.offset = 0;
 	    for (node = _jitc->head; node; node = node->next) {
 		if (node->link &&
 		    (node->code == jit_code_label ||
 		     node->code == jit_code_epilog))
 		    node->flag &= ~jit_flag_patch;
 	    }
+	    if (_jit->user_code)
+		goto fail;
 #if GET_JIT_SIZE
 	    ++_jitc->mult;
 	    length = _jitc->pool.length * 1024 * _jitc->mult;
@@ -1759,25 +1888,35 @@ _jit_emit(jit_state_t *_jit)
 	    _jitc->code.end = _jit->code.ptr + _jit->code.length -
 		jit_get_max_instr();
 	    _jit->pc.uc = _jit->code.ptr;
-	    _jitc->patches.offset = 0;
 	}
 	else
 	    break;
     }
 
 #if defined(__sgi)
-    close(mmap_fd);
+    if (!_jit->user_code)
+	close(mmap_fd);
 #endif
 
     _jitc->done = 1;
-    jit_annotate();
+    if (!_jitc->no_note)
+	jit_annotate();
 
-    result = mprotect(_jit->data.ptr, _jit->data.length, PROT_READ);
-    assert(result == 0);
-    result = mprotect(_jit->code.ptr, _jit->code.length, PROT_READ | PROT_EXEC);
-    assert(result == 0);
+    if (_jit->user_data)
+	jit_free((jit_pointer_t *)&_jitc->data.ptr);
+    else {
+	result = mprotect(_jit->data.ptr, _jit->data.length, PROT_READ);
+	assert(result == 0);
+    }
+    if (!_jit->user_code) {
+	result = mprotect(_jit->code.ptr, _jit->code.length,
+			  PROT_READ | PROT_EXEC);
+	assert(result == 0);
+    }
 
     return (_jit->code.ptr);
+fail:
+    return (NULL);
 }
 
 /*   Compute initial reglive and regmask set values of a basic block.
